from django.urls import path
from django.views.decorators.csrf import csrf_exempt

from .views import AuthorizationView, ProfileView

app_name = "social_oauth_token"

urlpatterns = [
    path("profile/", ProfileView.as_view(), name="profile"),
    path("token/<str:backend>/", csrf_exempt(AuthorizationView.as_view()), name="token"),
]



from datetime import timedelta

from django.contrib.auth import login
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import HttpResponse, JsonResponse
from django.utils import timezone
from django.views.generic import TemplateView, View
from oauth2_provider.models import (
    get_access_token_model,
    get_application_model,
    get_refresh_token_model,
)
from oauth2_provider.settings import oauth2_settings
from oauthlib import common
from social_core.exceptions import AuthCanceled, MissingBackend
from social_django.utils import load_backend, load_strategy

Application = get_application_model()


class ProfileView(LoginRequiredMixin, TemplateView):
    def get(self, request, *args, **kwargs):
        return HttpResponse()


class AuthorizationView(View):
    def post(self, request, backend, *args, **kwargs):
        try:
            client_id = request.POST.get("client_id", None)
            user = self.get_user(request, backend)

            if backend and request.POST.get("code", None) and client_id and user:
                login(request, user)
                request.session.set_expiry(0)  # expire when the Web browser is closed
                return self.create_access_token(user, client_id)
        except MissingBackend:
            pass
        return JsonResponse({"message": "Bad Request"}, status=400)

    def get_user(self, request, backend_name: str):
        try:
            backend = load_backend(load_strategy(request), backend_name, None)

            # Backend -c шалтгаалан authorizationCode -г баталгаажуулах процесс
            # нь redirect хийхгүй учир state -г ашиглах шаардлагагүй

            if getattr(backend, "STATE_PARAMETER", False):  # pragma: no cover
                backend.STATE_PARAMETER = False

            return backend.complete()
        except AuthCanceled:
            return None

    def create_access_token(self, user, client_id: str):
        try:
            application = Application.objects.get(
                client_id=client_id,
                client_type=Application.CLIENT_PUBLIC,
            )
            access_token = get_access_token_model().objects.create(
                user=user,
                expires=timezone.now() + timedelta(seconds=oauth2_settings.defaults["ACCESS_TOKEN_EXPIRE_SECONDS"]),
                token=common.generate_token(),
                application=application,
            )
            refresh_token = get_refresh_token_model().objects.create(
                user=user,
                token=common.generate_token(),
                application=application,
                access_token=access_token,
            )

            return JsonResponse(
                {
                    "access_token": access_token.token,
                    "expires_in": access_token.expires,
                    "token_type": "Bearer",
                    "refresh_token": refresh_token.token,
                }
            )
        except Application.DoesNotExist as e:
            return JsonResponse({"message": str(e)}, status=403)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['social_oauth_token', 'social_oauth_token.migrations']

package_data = \
{'': ['*']}

install_requires = \
['django-oauth-toolkit>=1.5.0,<2.0.0', 'social-auth-app-django>=5.0.0,<6.0.0']

setup_kwargs = {
    'name': 'django-social-oauth-token',
    'version': '0.1.0',
    'description': 'OAuth Token generation API for handling OAuth 2.0 Authentication Code Flow based on social-auth',
    'long_description': '<h1 align="center">\n  django-social-oauth-token\n</h1>\n\n<p align="center">\n  <a href="https://github.com/khasbilegt/django-social-oauth-token/">\n    <img src="https://img.shields.io/github/workflow/status/khasbilegt/django-social-oauth-token/CI?label=CI&logo=github&style=for-the-badge" alt="ci status">\n  </a>\n  <a href="https://pypi.org/project/django-social-oauth-token/">\n    <img src="https://img.shields.io/pypi/v/django-social-oauth-token?style=for-the-badge" alt="pypi link">\n  </a>\n  <a href="https://codecov.io/github/khasbilegt/django-social-oauth-token">\n    <img src="https://img.shields.io/codecov/c/github/khasbilegt/django-social-oauth-token?logo=codecov&style=for-the-badge" alt="codecov">\n  </a>\n  <br>\n  <a>\n    <img src="https://img.shields.io/pypi/pyversions/django-social-oauth-token?logo=python&style=for-the-badge" alt="supported python versions">\n  </a>\n  <a>\n    <img src="https://img.shields.io/pypi/djversions/django-social-oauth-token?logo=django&style=for-the-badge" alt="supported django versions">\n  </a>\n</p>\n\n<p align="center">\n  <a href="#installation">Installation</a> •\n  <a href="#contributing">Contributing</a> •\n  <a href="#how-to-use">How To Use</a> •\n  <a href="#license">License</a>\n</p>\n\n<p align="center">OAuthToken generation API for handling OAuth 2.0 Authentication Code Flow based on social-auth</p>\n\n## Installation\n\n1. Use your preferred package manager ([pip](https://pip.pypa.io/en/stable/), [poetry](https://pypi.org/project/poetry/), [pipenv](https://pypi.org/project/pipenv/)) to install the package. For example:\n\n```bash\n$ poetry add django-social-oauth-token\n```\n\n2. Then register \'social_oauth_token\', in the \'INSTALLED_APPS\' section of your project\'s settings.\n\n```python\n# settings.py\n...\n\nINSTALLED_APPS = (\n    ...\n    \'social_oauth_token\',\n)\n\n...\n```\n\n## How To Use\n\nIn order to verify the **Authorization Code** sent by the user and replace it with your own **OAuth Access Token**, send a **POST** request to the `token/<backend>/` endpoint with `client_id` and `code` to receive the token.\n\nThe POST request parameters:\n\n```Python\nclient_id # OAuth Client ID\ncode # Authorization Code\n```\n\nThe JSON response:\n\n```json\n{\n  "access_token": <access_token>,\n  "expires_in": <expires_in>,\n  "token_type": <token_type>,\n  "refresh_token": <refresh_token>,\n}\n```\n\n## Contributing\n\nPull requests are welcome. For major changes, please open an issue first to discuss what you would like to change.\n\nPlease make sure to update tests as appropriate.\n\n## License\n\n[MIT License](https://choosealicense.com/licenses/mit/)\n',
    'author': 'Khasbilegt.TS',
    'author_email': 'khasbilegt.ts@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/khasbilegt/django-social-oauth-token',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)
